@@ -1,13 +1,17 @@
 package Data::Transmute;
 
-our $DATE = '2019-10-10'; # DATE
-our $VERSION = '0.036'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2020-02-13'; # DATE
+our $DIST = 'Data-Transmute'; # DIST
+our $VERSION = '0.037'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
 use warnings;
 use Log::ger;
 
+use Scalar::Util qw(refaddr);
+
 use Exporter qw(import);
 our @EXPORT_OK = qw(transmute_data reverse_rules);
 
@@ -23,14 +27,16 @@ sub _rule_create_hash_key {
         return if $args{ignore};
         die "Rule create_hash_key: Key '$name' already exists" unless $args{replace};
     }
-    die "Rule create_hash_key: Please specify 'value'" unless exists $args{value};
-    $data->{$name} = $args{value};
+    die "Rule create_hash_key: Please specify 'value' or 'value_code'"
+        unless exists $args{value} || $args{value_code};
+    $data->{$name} = $args{value_code} ? $args{value_code}->($data->{$name}) : $args{value};
 }
 
 sub _rulereverse_create_hash_key {
     my %args = @_;
-    die "Cannot generate reverse rule create_hash_key with ignore=1"  if $args{ignore};
-    die "Cannot generate reverse rule create_hash_key with replace=1" if $args{replace};
+    die "Cannot generate reverse rule create_hash_key with value_code" if $args{value_code};
+    die "Cannot generate reverse rule create_hash_key with ignore=1"   if $args{ignore};
+    die "Cannot generate reverse rule create_hash_key with replace=1"  if $args{replace};
     [delete_hash_key => {name=>$args{name}}];
 }
 
@@ -80,11 +86,12 @@ sub _rule_modify_hash_value {
     my $from = $args{from};
     my $from_exists = exists $args{from};
     my $to   = $args{to};
-    die "Rule rename_hash_key: Please specify 'to'" unless exists $args{to};
+    die "Rule rename_hash_key: Please specify 'to' or 'to_code'"
+        unless exists $args{to} || $args{to_code};
 
     my $errprefix = "Rule modify_hash_value: Can't modify key '$name'".
         ($from_exists ? " from '".($from // '<undef>') : "").
-        "' to '".($to // '<undef>')."'";
+        ($args{to_code} ? "' using to_code" : "' to '".($to // '<undef>')."'");
 
     unless (exists $data->{$name}) {
         die "$errprefix: key does not exist";
@@ -92,6 +99,8 @@ sub _rule_modify_hash_value {
 
     my $cur = $data->{$name};
 
+    $to = $args{to_code}->($cur) if $args{to_code};
+
     if ($from_exists) {
         # noop
         return unless Data::Cmp::cmp_data($from, $to);
@@ -107,6 +116,7 @@ sub _rule_modify_hash_value {
 sub _rulereverse_modify_hash_value {
     my %args = @_;
     die "Cannot generate reverse rule modify_hash_value without from" unless exists $args{from};
+    die "Cannot generate reverse rule modify_hash_value with to_code" if $args{to_code};
     [modify_hash_value => {
         name => $args{name}, from => $args{to}, to => $args{from},
     }];
@@ -223,6 +233,47 @@ sub _rulereverse_transmute_hash_values {
     }];
 }
 
+sub _walk {
+    my ($data, $rule_args, $seen) = @_;
+
+    # transmute the node itself
+    transmute_data(
+        data => $data,
+        (rules        => $rule_args->{rules})        x !!(exists $rule_args->{rules}),
+        (rules_module => $rule_args->{rules_module}) x !!(exists $rule_args->{rules_module}),
+    );
+    my $ref = ref($data) or return;
+    my $refaddr = refaddr($data);
+    return if $seen->{$refaddr}++;
+
+    if ($ref eq 'ARRAY') {
+        for my $elem (@$data) {
+            _walk($elem, $rule_args, $seen);
+        }
+    } elsif ($ref eq 'HASH') {
+        for my $key (sort keys %$data) {
+            _walk($data->{$key}, $rule_args, $seen);
+        }
+    }
+}
+
+sub _rule_transmute_nodes {
+    my %args = @_;
+
+    my $data = $args{data};
+
+    die "Rule transmute_nodes: Please specify 'rules' or 'rules_module'"
+        unless defined($args{rules}) || defined($args{rules_module});
+
+    my $seen = {};
+    _walk($data, \%args, $seen);
+    $data;
+}
+
+sub _rulereverse_transmute_nodes {
+    die "Rule transmute_nodes is not reversible";
+}
+
 sub _rules_or_rules_module {
     my $args = shift;
 
@@ -296,7 +347,7 @@ Data::Transmute - Transmute (transform) data structure using rules data
 
 =head1 VERSION
 
-This document describes version 0.036 of Data::Transmute (from Perl distribution Data-Transmute), released on 2019-10-10.
+This document describes version 0.037 of Data::Transmute (from Perl distribution Data-Transmute), released on 2020-02-13.
 
 =head1 SYNOPSIS
 
@@ -417,18 +468,34 @@ Known arguments (C<*> means required):
 
 =item * name*
 
-=item * value*
+=item * value
+
+Either C<value> or C<value_code> is required.
+
+=item * value_code
+
+Either C<value> or C<value_code> is required.
+
+Instead of specifying value, you can also supply a coderef to compute the value.
+The coderef will be passed the current value of the hash key (or undef if there
+is none).
+
+If you supply C<value_code>, your rule will become irreversible.
 
 =item * ignore
 
 Bool. If set to true, will ignore/noop if key already exists. This is like
 INSERT IGNORE (INSERT OR IGNORE) in SQL.
 
+If you set C<ignore> to true, your rule will become irreversible.
+
 =item * replace
 
 Bool. If set to true, will replace existing keys. This is like REPLACE INTO in
 SQL.
 
+If you set C<replace> to true, your rule will become irreversible.
+
 =back
 
 =head2 rename_hash_key
@@ -474,10 +541,20 @@ String. Key name.
 
 String or undef. Original value.
 
-=item * to*
+=item * to
 
 String or undef. New value.
 
+Either C<to> or C<to_code> is required.
+
+=item * to_code
+
+Coderef. Instead of specifying new vlaue via C<to>, you can also supply
+C<to_code> (a coderef) to compute the value. The coderef will be passed the
+current value.
+
+If you set C<to_code>, your rule will become irreversible.
+
 =back
 
 =head2 delete_hash_key
@@ -559,6 +636,30 @@ C<hash> (the hash).
 
 =back
 
+=head2 transmute_nodes
+
+This rule will transmute data, then recurse (walk) to array elements (if data is
+an array) or hash values (if data is a hash) and transmute each of those child
+data. Can handle circular references.
+
+This rule is not irreversible.
+
+Known arguments (C<*> means required):
+
+=over
+
+=item * rules
+
+Either C<rules> or C<rules_module> is required. C<rules> takes precedence over
+C<rules_module>.
+
+=item * rules_module
+
+Either C<rules> or C<rules_module> is required. C<rules> takes precedence over
+C<rules_module>.
+
+=back
+
 =head1 FUNCTIONS
 
 =head2 transmute_data
@@ -698,7 +799,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019, 2015 by perlancar@cpan.org.
+This software is copyright (c) 2020, 2019, 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
